Abstract Machine for Bottom-Up Evaluation in Deductive DBs with the Push Method
===============================================================================

This program is under development. It is written in C++ by Stefan Brass.
EMail: brass@informatik.uni-halle.de

Contents of directories:
-----------------------
- base:  Basic Definitions and Programming Support
- mem:   Memory management
- bds:   Basic data structures (stack and flexible array)
- dom:   Different implementations of string tables (for domains)
- rel:   Different implementations of relations (lists, sets, maps)
- input: Input sources (e.g. for reading files)
- pred:  Database predicates linked to the corresponding relations
- facts: Data loader for facts
- emu:   Emulator (Interpreter) for Bottom-Up Abstract Machine
- test:  Main program and performance tests (benchmarks)
- data:  Test data files

Installing the software:
-----------------------
- Clone the git repository:  
  `git clone git@gitlab.informatik.uni-halle.de:brass/bam.git`
- [CMake](https://cmake.org/) is used to create a Makefile from CMakeLists.txt  
  Running `cmake` without parameters prints the available generators.  
  For instance, to generate UNIX makefiles, run  
     `cmake -G "Unix Makefiles" .`  
  This assumes that the current directory `.` is where the sources are stored.
  CMake can also generate project files for Microsoft Visual C++
  and other IDEs.
- Then run `make` on UNIX, or use your IDE to build the project.
- The data files are not part of the git repository.
  Create a directory `data` and download the needed files from
  [http://users.informatik.uni-halle.de/~brass/push/data/].
  For the transitive closure benchmark, you will need the files `tc_*`.
- Documentation can be generated with Doxygen:  
  [http://www.stack.nl/~dimitri/doxygen/]
  Just run `doxygen` in the main directory.
  The generated documentation will be stored in the directory `doc`.
  One can start by opening `doc\html\index.html`.
  Currently, not all classes and methods are fully documented,
  but the generated documentation should be useful.
 


Executing the Program:
---------------------
- Usage: `./bam <Options> <Tests>`
- Options are:
    * `-aN`: Show the first N answers of the benchmark query.
    * `-a`:  Do not show answers (default are the first 10 answers).
    * `-d`:  Print additional debug output (includes data structure dumps).
    * `-l`:  List all tests.
    * `-m`:  Show memory pools.
    * `-r`:  List relations.
    * `-s`:  Print only summary line for test/benchmark ("silent").
    * `-b`:  For benchmarks: Similar to -s, no newline at end, msg in debug ver.
- To see a list of all tests, call `./bam -l`
- E.g. `./bam -b tcff_5_f2` executes the TCFF benchmark  
  with implementation 5 (the abstract machine)  
  for data file 2: `../data/tc_d1000_parsize50000_xsb_cyc.P`

