// ============================================================================
// Project:	Deductive Database
// Filename:	inst.cpp
// Purpose:	Enumeration Type for Abstract Machine Instructions (OpCodes)
// Last Change:	19.09.2017
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2017 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Interface:
//=============================================================================

#include "../base/ver.h"
#include "../base/str.h"
#include "../base/check.h"

#include "itype.h"

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// itype_valid: Is the parameter a valid element of the enumeration type?
//-----------------------------------------------------------------------------

#if	VER_DEBUG

bool	itype_valid(itype_t itype)
{
	return(
		// Instruction with no arguments:
		itype == ITYPE_NONE ||

		// Instruction with one 8 bit argument:
		itype == ITYPE_8 ||

		// Instruction with one 16 bit argument:
		itype == ITYPE_16 ||

		// Instruction with two 8 bit arguments:
		itype == ITYPE_8_8 ||

		// Instruction with three 8 bit arguments:
		itype == ITYPE_8_8_8 ||

		// Instruction with one 8 bit argument and one 16 bit argument:
		itype == ITYPE_8_16 ||

		// Instruction with three arguments (8 bit, 16 bit, 8 bit):
		itype == ITYPE_8_16_8

	);
}

#endif


//-----------------------------------------------------------------------------
// itype_name: String representation of this instruction type.
//-----------------------------------------------------------------------------

str_t	itype_name(itype_t itype)
{
	str_t name;
	
	switch(itype) {
		// Instruction with no arguments:
		case ITYPE_NONE:
			name = "ITYPE_NONE";
			break;

		// Instruction with one 8 bit argument:
		case ITYPE_8:
			name = "ITYPE_8";
			break;

		// Instruction with one 16 bit argument:
		case ITYPE_16:
			name = "ITYPE_16";
			break;

		// Instruction with two 8 bit arguments:
		case ITYPE_8_8:
			name = "ITYPE_8_8";
			break;

		// Instruction with three 8 bit arguments:
		case ITYPE_8_8_8:
			name = "ITYPE_8_8_8";
			break;

		// Instruction with one 8 bit argument and one 16 bit argument:
		case ITYPE_8_16:
			name = "ITYPE_8_16";
			break;

		// Instruction with three arguments (8 bit, 16 bit, 8 bit):
		case ITYPE_8_16_8:
			name = "ITYPE_8_16_8";
			break;

		default:
			CHECK_IMPOSSIBLE("itype_name: unknown type");
			name = "? (unknown/illegal)";
			break;
	}
	
	return name;
}

