// ============================================================================
// Project:	Deductive Database
// Filename:	set_s.cpp
// Purpose:	Set of tuples (binding pattern b..b): Using C++11 Standard Lib
// Last Change:	04.08.2017
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2016-2017 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Interface:
//=============================================================================

#include "../base/ver.h"
#include "../base/str.h"
#include "../base/err.h"
#include "../base/check.h"

#include "set_s.h"


//=============================================================================
// Constructor and Destructor:
//=============================================================================


//-----------------------------------------------------------------------------
// Constructor:
//-----------------------------------------------------------------------------

template<class T> set_s_c<T>::set_s_c(str_t rel_name) :
				rel_c(rel_name, T::NUM_COLS, 0,
					"Standard Library unordered_set")
{
	// Set Magic number:
	CHECK_CODE(Magic = SET_S_MAGIC);

	// Check parameters:
	CHECK(rel_name != STR_NULL,
			"set_s_c::constructor: rel_name is NULL");

}

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

template<class T> set_s_c<T>::~set_s_c()
{
	// Check this object:
	CHECK_VALID("set_s_c::destructor");

	// Make this object invalid:
	CHECK_CODE(Magic = 0);
}

//-----------------------------------------------------------------------------
// insert: Insert an element into the set (returns true if successful).
//-----------------------------------------------------------------------------

//template<class T> bool set_s_c<T>::insert(T elem)
//{
//}


//=============================================================================
// Debugging Support:
//=============================================================================

//-----------------------------------------------------------------------------
// check: Check the integrity of this object, return error message or NULL.
//-----------------------------------------------------------------------------

#if	VER_DEBUG
template<class T> str_t	set_s_c<T>::check() const
{
	// Check magic number:
	if(Magic != SET_S_MAGIC)
		return "wrong magic number";

	// Check superclass:
	str_t msg = rel_c::check();
	if(msg)
		return msg;

	// Ok:
	return 0;
}
#endif


//-----------------------------------------------------------------------------
// dump: Show data structure.
//-----------------------------------------------------------------------------

#if	VER_DUMP
template<class T> void set_s_c<T>::dump(str_t headline) const
{
	// Headline:
	if(headline == STR_NULL)
		headline =
		"Set: Relation with binding pattern b..b for Duplicate Check";
	err_c::dump_begin(headline);

	// Basic information:
	err_c::dump_str("Name:       '");
	err_c::dump_str(Name);
	err_c::dump_str("'");
	err_c::dump_nl();

	err_c::dump_str("NumRows:     ");
	err_c::dump_int(NumRows);
	err_c::dump_nl();


	err_c::dump_str("ObjectSize:  ");
	err_c::dump_uint(static_cast<unsigned int>(sizeof(set_s_c<T>)));
	err_c::dump_str(" Bytes");
	err_c::dump_nl();

	err_c::dump_nl();

	// Done:
	err_c::dump_end();
}
#endif

