// ============================================================================
// Project:	Deductive Database
// Filename:	load_tc.h
// Purpose:	Parse and load data for "parent" relation for tc benchmark.
// Last Change:	04.08.2017
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2017 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


/**
 * \file
 * \brief Data Loader for Transitive Closure Benchmark
 * 	(with Binding Pattern "ff")
 */


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	LOAD_TC_INCLUDED
#define LOAD_TC_INCLUDED


//=============================================================================
// This Module is optional:
//=============================================================================

#ifndef VER_INCLUDED
#include "../base/ver.h"
#endif

#if	VER_TCFF_LOAD


//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef STR_INCLUDED
#include "../base/str.h"
#endif

#ifndef CHECK_INCLUDED
#include "../base/check.h"
#endif

#ifndef	LIST_2_INCLUDED
#include "../rel/list_2.h"
#endif

#ifndef	REL_N_N_INCLUDED
#include "../rel/rel_n_n.h"
#endif

//=============================================================================
// Load Data, Version for "dblp.data" - Class With Only Static Members:
//=============================================================================

class load_tc_c {
	public:

//-----------------------------------------------------------------------------
// Static Methods:
//-----------------------------------------------------------------------------

	// Load data:
	static bool load(
			// Name of data file to load:
			str_t filename,
			// Relations for EDB-literals:
			list_2_t  par_ff,
			rel_n_n_t par_fb);

//-----------------------------------------------------------------------------
// This class has no objects:
//-----------------------------------------------------------------------------

	private:

	// Constructor:
	load_tc_c();	// Not implemented

	// Destructor:
	~load_tc_c();	// Not implemented

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	load_tc_c(const load_tc_c& obj);		// Not implemented
	load_tc_c& operator=(const load_tc_c& obj);	// Not implemented


//=============================================================================
// End of Class:
//=============================================================================

};


//=============================================================================
// End of Conditional Compilation of this Test (VER_TCFF_LOAD):
//=============================================================================

#endif


//=============================================================================
// End of Include File:
//=============================================================================

#endif

